﻿using System;

// http://dotnetfollower.com
namespace dotNetFollower
{
    /// <summary>
    /// Represents current state of an async job
    /// </summary>
    [Serializable]
    public class AsyncJobState : AsyncOperationState<AsyncOperationState?, asyncoperation_statuscode?>
    {
        /// <summary>
        /// Id of the async job
        /// </summary>
        public Guid AsyncJobId { get; set; }

        /// <summary>
        /// Indicates if the async job is suspended
        /// </summary>
        public override bool IsSuspended
        {
            get { return CurrentState != null && CurrentState.Value == AsyncOperationState.Suspended; }
        }
        /// <summary>
        /// Indicates if the async job is paused
        /// </summary>
        public override bool IsPaused
        {
            get { return CurrentState != null && CurrentState.Value == AsyncOperationState.Locked && CurrentStatus != null && CurrentStatus.Value == asyncoperation_statuscode.Pausing; }
        }
        /// <summary>
        /// Indicates if the async job is completed
        /// </summary>
        public override bool IsCompleted
        {
            get { return CurrentState != null && CurrentState.Value == AsyncOperationState.Completed; }
        }
        /// <summary>
        /// Indicates if the number of retries have run out while the async job is still uncompleted
        /// </summary>
        public override bool IsTimedOut
        {
            get { return !IsCompleted && CurrentRetryCount <= 0; }
        }
        /// <summary>
        /// Indicates if the async job is failed
        /// </summary>
        public override bool IsFailed
        {
            get { return CurrentStatus != null && CurrentStatus.Value == asyncoperation_statuscode.Failed; }
        }
    }
}
