﻿using System;

// http://dotnetfollower.com
namespace dotNetFollower
{
    /// <summary>
    /// Comprises the awaiting parameters and id of an operation to be tracked. 
    /// Provides the OnProgress event supposed to be fired on each iteration of awaiting process.
    /// </summary>
    /// <typeparam name="TState">Represents a type of the operation state</typeparam>
    /// <typeparam name="TParams">Represents a type of the awaiting parameters</typeparam>
    public abstract class AsyncOperationAwaiting<TState, TParams>
    {
        protected readonly Guid    _asyncOperationId;
        protected readonly TParams _asyncOperationAwaitingParams;

        /// <summary>
        /// Represents the awaiting parameters
        /// </summary>
        public TParams Params
        {
            get { return _asyncOperationAwaitingParams; }
        }

        /// <summary>
        /// Event to fire and get operation state on each iteration
        /// </summary>
        public event EventHandler<TState> OnProgress;

        public void FireOnProgress(TState state)
        {
            if (OnProgress != null)
                OnProgress(this, state);
        }

        protected AsyncOperationAwaiting(Guid asyncOperationId, TParams asyncOperationAwaitingParams)
        {
            _asyncOperationId             = asyncOperationId;
            _asyncOperationAwaitingParams = asyncOperationAwaitingParams;
        }
    }
}
