﻿#define WRITE_TRACE_INFO

using System;
using System.Threading;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Xrm.Sdk.Query;

// http://dotnetfollower.com
namespace dotNetFollower
{
    public static class CrmHelper
    {
        public static AsyncJobState WaitForAsyncJobCompletion(OrganizationServiceProxy proxy, AsyncJobAwaiting asyncJobAwaiting)
        {
            var state = new AsyncJobState();

            #if WRITE_TRACE_INFO
            WriteInfo(string.Format("Waiting for async job (Id: {0}) completion ...", asyncJobAwaiting.AsyncJobId));
            #endif

            var columnSet = new ColumnSet("statecode", "statuscode");

            state.AsyncJobId        = asyncJobAwaiting.AsyncJobId;
            state.CurrentRetryCount = asyncJobAwaiting.Params.RetryCount;
            state.CurrentTimeout    = asyncJobAwaiting.Params.StartTimeout;

            do
            {
                Thread.Sleep(state.CurrentTimeout);

                var asyncJob = (AsyncOperation)proxy.Retrieve("asyncoperation", asyncJobAwaiting.AsyncJobId, columnSet);

                state.CurrentState  = asyncJob.StateCode;
                state.CurrentStatus = asyncJob.StatusCode != null ? (asyncoperation_statuscode)asyncJob.StatusCode.Value : (asyncoperation_statuscode?)null;
                
                #if WRITE_TRACE_INFO
                if (state.IsSuspended)
                    WriteInfo(string.Format("Async job (Id: {0}) is suspended", asyncJobAwaiting.AsyncJobId));
                if (state.IsPaused)
                    WriteInfo(string.Format("Async job (Id: {0}) is paused", asyncJobAwaiting.AsyncJobId));
                if (asyncJob.StateCode != null)
                    WriteInfo(string.Format("Async job (Id: {0}) state is {1} {2}", asyncJobAwaiting.AsyncJobId, asyncJob.StateCode.Value, (asyncJob.StatusCode != null ? "(Status: " + (asyncoperation_statuscode)asyncJob.StatusCode.Value + ")" : string.Empty)));
                #endif

                asyncJobAwaiting.FireOnProgress(state);

                if (asyncJobAwaiting.Params.TimeoutStep != null && asyncJobAwaiting.Params.EndTimeout != null && state.CurrentTimeout < asyncJobAwaiting.Params.EndTimeout)
                    state.CurrentTimeout += asyncJobAwaiting.Params.TimeoutStep.Value;

                if (asyncJobAwaiting.Params.WaitForever)
                    continue;
                if (state.IsSuspended && asyncJobAwaiting.Params.DoNotCountIfSuspended)
                    continue;
                if (state.IsPaused && asyncJobAwaiting.Params.DoNotCountIfPaused)
                    continue;

                state.CurrentRetryCount--;

            } while (!state.IsCompleted && state.CurrentRetryCount > 0);

            return state;
        }

        public static void WriteInfo(string message)
        {
            Console.WriteLine(message);
        }

        public static void WaitForAsyncJobCompletion(OrganizationServiceProxy serviceProxy, Guid asyncJobId)
        {
            ColumnSet cs = new ColumnSet("statecode", "statuscode");
            AsyncOperation asyncjob = (AsyncOperation)serviceProxy.Retrieve("asyncoperation", asyncJobId, cs);

            int retryCount = 100;

            while (asyncjob.StateCode.Value != AsyncOperationState.Completed && retryCount > 0)
            {
                asyncjob = (AsyncOperation)serviceProxy.Retrieve("asyncoperation", asyncJobId, cs);
                System.Threading.Thread.Sleep(2000);
                retryCount--;
                Console.WriteLine("Async operation state is " + asyncjob.StateCode.Value.ToString());
            }

            Console.WriteLine("Async job is " + asyncjob.StateCode.Value.ToString() + 
                " with status " + ((asyncoperation_statuscode)asyncjob.StatusCode.Value).ToString());
        }
    }
}
